<?php

namespace RUMCWV;

if (!defined('ABSPATH')) {
    die('Let\'s not do this');
}

/** 
 * RUM TRACKING, mostly self contained rum class
 * There is some rum stuff left in the settings class
 * @todo, truncate data after xx days
 **/



class RUMCWV_Rum
{

    public static function init()
    {

        if (is_admin() || (function_exists('is_login') && is_login())) {
            self::backend_actions();
        } else {
            // 2 reasons not to track
            if (!defined('DONOTCACHEPAGE')) {
                // do not track admin
                if (!\current_user_can('manage_options')) {
                    self::trackingscript();
                }
            }
        }
    }

    private static function trackingscript()
    {

        $pct = (int)RUMCWV_Options::get('rum_cwv_sample_rate');
        if ($pct <= 0 || $pct > 100) {
            $pct = 100;
        }
        $sample_rate = $pct / 100;

        $rum_cwv_endpoint = json_encode(admin_url('admin-ajax.php'));

        $script = 'var rum_cwv_endpoint=' . $rum_cwv_endpoint . ';' . $sample_rate . '>Math.random()&&window.addEventListener("load",function(){var e=document.createElement("script");e.src="' . RUM_CWV_PLUGIN_DIR . 'public/js/rum.js?ver=' . RUM_CWV_VERSION . '",document.body.appendChild(e)});';

        wp_register_script('rum_cwv_rum_tracking', '', [], '', true);
        wp_enqueue_script('rum_cwv_rum_tracking');
        wp_add_inline_script('rum_cwv_rum_tracking', $script);
    }


    private static function backend_actions()
    {

        // actions
        add_action('wp_ajax_rum_cwv_cache', 'RUMCWV\RUMCWV_Rum::admin_ajax_rum_cwv_cache');
        add_action('wp_ajax_cwv_get_progress', 'RUMCWV\RUMCWV_Rum::admin_ajax_rumcwv_get_progress');
        add_action('wp_ajax_cwv_get_p75', 'RUMCWV\RUMCWV_Rum::admin_ajax_rumcwv_get_p75');
        add_action('wp_ajax_cwv_get_histogram', 'RUMCWV\RUMCWV_Rum::admin_ajax_rumcwv_get_histogram');
        add_action('wp_ajax_cwv_get_timeline', 'RUMCWV\RUMCWV_Rum::admin_ajax_rumcwv_get_timeline');
        add_action('wp_ajax_cwv_get_urllist', 'RUMCWV\RUMCWV_Rum::admin_ajax_rumcwv_get_urllist');

        add_action('wp_ajax_cwv_truncate_rum', 'RUMCWV\RUMCWV_Rum::admin_ajax_rumcwv_truncate_rum');
        add_action('wp_ajax_cwv_tmp_to_main', 'RUMCWV\RUMCWV_Rum::admin_ajax_rumcwv_tmp_to_main');

        add_action('wp_ajax_collect_rumcwv', 'RUMCWV\RUMCWV_Rum::admin_ajax_collect_rumcwv');
        add_action('wp_ajax_nopriv_collect_rumcwv', 'RUMCWV\RUMCWV_Rum::admin_ajax_nopriv_collect_rumcwv');
    }

    /**
     * precache rum data (called from cron)
     */
    public static function precache()
    {
        $data = [];
        foreach (RUMCWV_RumDB::$aMetrics as $metric) {
            $device = self::_validateDevice($_POST['d']);
            $data[$metric]['timeline'] = RUMCWV_RumDB::timeline($metric,$device);
            $data[$metric]['histogram'] = RUMCWV_RumDB::histogram($metric,$device);
            $data[$metric]['progress'] = RUMCWV_RumDB::progress($metric,$device);
            $p75 = RUMCWV_RumDB::percentile($metric, 75);
            $data[$metric]['p75'] = $p75;
        }
        // save to transient
        set_transient('rum_cwv_cache', json_encode($data), 60 * 60 * 24);
    }

    // ajax admin function get rum data from cache
    public static function admin_ajax_rum_cwv_cache()
    {
        // get transient
        $data = get_transient('rum_cwv_cache');
        if (!$data || $_POST['d'] != 'all') {
            $data = json_encode([]);
        }

        $data = json_decode($data, true);
        echo wp_json_encode($data);
        exit();
    }


    // ajax admin function clear all rum data
    public static function admin_ajax_rumcwv_truncate_rum()
    {
        RUMCWV_RumDB::truncate();

        // clear transient
        delete_transient('rum_cwv_cache');

        echo wp_json_encode(['success' => true]);
        exit();
    }

    // ajax admin function get progress for metric
    public static function admin_ajax_rumcwv_tmp_to_main()
    {
        RUMCWV_RumDB::tmp_to_main();
        RUMCWV_Rum::precache();
        echo wp_json_encode(['success' => true]);
        exit();
    }

    // ajax admin function get progress for metric
    public static function admin_ajax_rumcwv_get_progress()
    {
        $device = self::_validateDevice($_POST['d']);
        $metric = self::_validateMetric($_POST['metric']);
        $bucket = RUMCWV_RumDB::progress($metric,$device);
        echo wp_json_encode($bucket);
        exit();
    }

    // ajax admin function get p75 for metric
    public static function admin_ajax_rumcwv_get_p75()
    {

        $device = self::_validateDevice($_POST['d']);
        $metric = self::_validateMetric($_POST['metric']);
        $p75 = RUMCWV_RumDB::percentile($metric,$device, 75);
        echo wp_json_encode(['p75' => $p75]);
        exit();
    }
    // ajax admin function get p75 for metric
    public static function admin_ajax_rumcwv_get_histogram()
    {
        $device = self::_validateDevice($_POST['d']);

        $metric = self::_validateMetric($_POST['metric']);
        $histogram = RUMCWV_RumDB::histogram($metric,$device);
        echo wp_json_encode($histogram);
        exit();
    }
    // ajax admin function get p75 for metric for timeline
    public static function admin_ajax_rumcwv_get_timeline()
    {
        $device = self::_validateDevice($_POST['d']);
        $metric = self::_validateMetric($_POST['metric']);
        $timeline = RUMCWV_RumDB::timeline($metric,$device);
        echo wp_json_encode($timeline);
        exit();
    }


    // ajax admin function get p75 for metric for each url
    public static function admin_ajax_rumcwv_get_urllist()
    {
        $metric = self::_validateMetric($_POST['metric']);
        $device = self::_validateDevice($_POST['d']);

        $urllist = RUMCWV_RumDB::urllist($metric,$device);
        echo wp_json_encode($urllist);
        exit();
    }


    private static function _validateMetric($metric)
    {
        if (!in_array($metric, RUMCWV_RumDB::$aMetrics)) {
            wp_die( 'Invalid data' );
        }

        return $metric;
    }

    private static function _validateDevice($device)
    {
        if (!in_array($device,["desktop","mobile","all"])) {
            wp_die( 'Invalid data' );
        }

        return $device;
    }


    public static function admin_ajax_collect_rumcwv()
    {
        echo wp_json_encode(['thank' => 'you']);
        exit();
    }

    /** 
     * save the rum data
     * @todo, save query string!
     **/
    public static function admin_ajax_nopriv_collect_rumcwv()
    {

        $data = json_decode(stripslashes($_POST['metrics']), true);

        foreach ($data as $d) {
            $id = $d['id'];
            $metricname = $d['n'];
            $metricvalue = $d['v'];
            // remove query string from $d['u']
            $url = strtok($d['u'], '?');
            $device = $d['d'];

            RUMCWV_RumDB::upsert($id, $metricname, $metricvalue, $url, $device);
        }
        echo wp_json_encode(['thank' => 'you']);
        exit();
    }
}
